local AddonName, Hermes = ...

function Hermes:InitializeHermesEvents()
	if not Hermes.Events then
		Hermes.Events = {}
		
		--communication
		Hermes.Events["OnStartSending"] = {}
		Hermes.Events["OnStopSending"] = {}
		Hermes.Events["OnStartReceiving"] = {}
		Hermes.Events["OnStopReceiving"] = {}
		
		--senders
		Hermes.Events["OnSenderAdded"] = {}
		Hermes.Events["OnSenderRemoved"] = {}
		Hermes.Events["OnSenderZoneChanged"] = {}
		Hermes.Events["OnSenderOnlineChanged"] = {}
		Hermes.Events["OnSenderDeadChanged"] = {}
		Hermes.Events["OnSenderAvailabilityChanged"] = {}
		
		--player
		Hermes.Events["OnPlayerGroupStatusChanged"] = {}
		Hermes.Events["OnPlayerZoneChanged"] = {}
		
		--abilities
		Hermes.Events["OnAbilityAdded"] = {}
		Hermes.Events["OnAbilityRemoved"] = {}
		Hermes.Events["OnAbilityAvailableSendersChanged"] = {}
		Hermes.Events["OnAbilityTotalSendersChanged"] = {}
		
		--ability instances
		Hermes.Events["OnAbilityInstanceAdded"] = {}
		Hermes.Events["OnAbilityInstanceRemoved"] = {}
		Hermes.Events["OnAbilityInstanceStartCooldown"] = {}
		Hermes.Events["OnAbilityInstanceUpdateCooldown"] = {}
		Hermes.Events["OnAbilityInstanceStopCooldown"] = {}
		Hermes.Events["OnAbilityInstanceAvailabilityChanged"] = {}
	end
end

function Hermes:UnregisterHermesEvent(event, key)
	--check for valid key
	if not key then
		error("hermes event key cannot be nil")
	end
	
	if Hermes.Events[event][key] then
		Hermes.Events[event][key] = nil
	end
end

function Hermes:RegisterHermesEvent(event, key, handler)
	--make sure that events are created
	self:InitializeHermesEvents()
	
	--check for valid key
	if not key then
		error("hermes event key cannot be nil")
	end
	
	--check for valid event
	if not Hermes.Events[event] then
		error("unknown hermes event: "..tostring(event))
	end
	
	--if nil handler, act like an unregister
	if not handler then
		self:UnregisterHermesEvent(event, key)
		return
	end
	
	--register the event, replaces any other registrations for the given ekey
	Hermes.Events[event][key] = handler
end

function Hermes:FireEvent(event, ...)
	--check for valid event
	if not Hermes.Events[event] then
		error("unknown hermes event: "..tostring(event))
	end
	
	--fire the events for each registered handler
	for key, handler in pairs(Hermes.Events[event]) do
		handler(...)
	end
end