local AddonName, Hermes = ...

-------------------------------------------------------------------------
--EVENTS
-------------------------------------------------------------------------
function Hermes:ZONE_CHANGED()
	self:UpdatePlayerZoneStatus()
end

function Hermes:ZONE_CHANGED_NEW_AREA()
	self:UpdatePlayerZoneStatus()
end

function Hermes:ZONE_CHANGED_INDOORS()
	self:UpdatePlayerZoneStatus()
end

function Hermes:InitializePlayer()
	Hermes.Player.name = UnitName("player")
	Hermes.Player.class = Hermes:GetClassEnum(select(2, UnitClass("player")))
	Hermes.Player.raid = false
	Hermes.Player.party = false
	Hermes.Player.battleground = false
	Hermes.Player.zone = GetRealZoneText()
end

function Hermes:UpdatePlayerZoneStatus()
	local oldZone = Hermes.Player.zone
	Hermes.Player.zone = GetRealZoneText()
	if oldZone ~= Hermes.Player.zone then --zone changed
	
		--fire off player zone changed event
		Hermes:FireEvent("OnPlayerZoneChanged", Hermes.Player.zone)
		
		--update available abilities since the players zone changed, could have knocked senders unavailable...
		--self:UpdateAbilityAvailabilityForAll()
		
		--...ditto for instances
		for _, instance in ipairs(Hermes.AbilityInstances) do
			--see if the sender when last checked belongs to the zone we were last in, or the zone we're currently in
			--ignore yourself, since zone will always match obvously!
			if(instance.sender.zone == oldZone or instance.sender.zone == Hermes.Player.zone and (instance.sender.name ~= Hermes.Player.name)) then
				--fire event on instance
				Hermes:FireEvent("OnAbilityTotalSendersChanged", instance.ability) -- ADDED
				Hermes:FireEvent("OnAbilityAvailableSendersChanged", instance.ability) -- ADDED
				Hermes:FireEvent("OnAbilityInstanceAvailabilityChanged", instance)
			end
		end
	end
end

function Hermes:UpdatePlayerGroupStatus()
	local wasInGroup = Hermes.Player.raid == true or Hermes.Player.party == true or Hermes.Player.battleground == true
	
	if(GetNumRaidMembers() > 0) then
		Hermes.Player.raid = true
	else
		Hermes.Player.raid = false
	end
	
	if(GetNumPartyMembers() > 0) then
		Hermes.Player.party = true
	else
		Hermes.Player.party = false
	end
	
	if(UnitInBattleground("player") ~= nil) then
		Hermes.Player.battleground = true
	else
		Hermes.Player.battleground = false
	end
	
	local isInGroup = Hermes.Player.party == true or Hermes.Player.raid == true or Hermes.Player.battleground == true
	
	if (wasInGroup == true and isInGroup == false) or (wasInGroup == false and isInGroup == true) then
		Hermes:FireEvent("OnPlayerGroupStatusChanged", isInGroup)
	end
end

