local AddonName, Hermes = ...

function Hermes:EnablePlugins()
	for name, plugin in pairs(Hermes.Plugins) do
		--only startup plugins that are enabled or don't exist in the profile (assumes they're new)
		if self.db.profile.pluginState[name] == nil then
			self.db.profile.pluginState[name] = true
		end
		if self.db.profile.pluginState[name] == true then
			self:EnablePlugin(name)
		end
	end
end

function Hermes:EnablePlugin(name)
	local plugin = Hermes.Plugins[name]
	if plugin.OnSetProfileCallback then
		plugin.OnSetProfileCallback(self.db.profile.plugins)
	end
	if plugin.OnEnableCallback then
		plugin.OnEnableCallback()
	end
end

function Hermes:DisablePlugins()
	for name, plugin in pairs(Hermes.Plugins) do
		self:DisablePlugin(name)
	end
end

function Hermes:DisablePlugin(name)
	local plugin = Hermes.Plugins[name]
	--only stop plugins that are enabled
	if self.db.profile.pluginState[name] == true and plugin.OnDisableCallback then
		plugin.OnDisableCallback()
	end
end

function Hermes:InitializeHermesPluginTable()
	if not Hermes.Plugins then
		Hermes.Plugins = {}
	end
end

function Hermes:RegisterHermesPlugin(name, OnEnableCallback, OnDisableCallback, OnSetProfileCallback, OnGetBlizzOptionsTable)
	--make sure that plugins table is created
	self:InitializeHermesPluginTable()
	
	if not name then
		error("plugin name cannot be nil")
	end
	
	--check to make sure it's not already registered
	if Hermes.Plugins[name] then
		error("plugin already registered: "..tostring(name))
	end
	
	--update the plugin state in the profile, default is enabled
	if self.db.profile.pluginState[name] == nil then
		self.db.profile.pluginState[name] = true
	end
	
	--register the plugin
	Hermes.Plugins[name] = {
		OnEnableCallback = OnEnableCallback,
		OnDisableCallback = OnDisableCallback,
		OnSetProfileCallback = OnSetProfileCallback,
		OnGetBlizzOptionsTable = OnGetBlizzOptionsTable,
	}
end